#include "widgetsimulationview.h"
#include <QDebug>

WidgetSimulationView::WidgetSimulationView(QWidget *parent) :
        QGLWidget(parent),
        modelObjsList(NULL)
{
    filter = 2;
    changeAmount = 0;
    xrot = yrot = 0.0f;
    xspeed = yspeed = 0.5f;
    z = -5.0f;
    quadricObj = NULL;
    object = 0;
    part1 = part2 = p1 = 0;
    p2 = 1;
}

void WidgetSimulationView::slotTime() {

    updateGL();

}

void WidgetSimulationView::initializeGL() {
    glShadeModel(GL_SMOOTH); // Enable smooth shading
    qglClearColor(Qt::black); // Set the clear color to a black background

    glClearDepth(1.0f); // Depth buffer setup
    glEnable(GL_DEPTH_TEST); // Enable depth testing
    glDepthFunc(GL_LEQUAL); // Set type of depth test

    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST); // Really nice perspective calculations

    // Set up textures
    // Note: Because we need different texture filters, we cannot use Qt's bindTexture()
    //       Qt automatically generates the most appropriate textures using available features in your setup
    //       which means that Qt always generates mipmaps if your system supports them
    //       We will use normal glBindTexture() etc. functions instead

    glEnable(GL_TEXTURE_2D);
    QImage img = convertToGLFormat(QImage(":/img/Wall.bmp"));
    glGenTextures(3, texture);

    // Texture using mipmaps
    glBindTexture(GL_TEXTURE_2D, texture[2]);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
    gluBuild2DMipmaps(GL_TEXTURE_2D, 3, img.width(), img.height(), GL_RGBA, GL_UNSIGNED_BYTE, img.bits());

    // Set up lighting
    GLfloat ambLight[] = {0.3f, 0.3f, 0.3f, 1.0f};
    GLfloat diffLight[] = {1.0f, 1.0f, 1.0f, 1.0f};
    GLfloat lightPos[] = {0.0f, 0.0f, 2.0f, 1.0f};
    glLightfv(GL_LIGHT1, GL_AMBIENT, ambLight);
    glLightfv(GL_LIGHT1, GL_DIFFUSE, diffLight);
    glLightfv(GL_LIGHT1, GL_POSITION, lightPos);
    glEnable(GL_LIGHT1);
    glEnable(GL_LIGHTING);

    // Set up quadric object
    quadricObj = gluNewQuadric();
    gluQuadricNormals(quadricObj, GLU_SMOOTH);
    gluQuadricTexture(quadricObj, GL_TRUE);
}

// OpenGL painting code goes here
void WidgetSimulationView::paintGL() {
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // Clear screen and depth buffer

    glLoadIdentity(); // Reset current modelview matrix
    glTranslatef(0.0f, 0.0f, z); // Move into the screen
//   glRotatef(xrot, 1.0f, 0.0f, 0.0f); // Rotate on X-axis
//   glRotatef(yrot, 0.0f, 1.0f, 0.0f); // Rotate on Y-axis
    glBindTexture(GL_TEXTURE_2D, texture[filter]);
    // MODEL PAINTING GOES HERE
    glBegin(GL_QUADS);
            glTranslatef(0.0f, 0.0f, -5.0); // Move into the screen
            // Front Face
            glNormal3f(0.0f, 0.0f, 1.0f);
            glVertex3f(-1.0f, -1.0f, 1.0f); // Bottom Left Of The Texture and Quad
            glVertex3f( 1.0f, -1.0f, 1.0f); // Bottom Right Of The Texture and Quad
            glVertex3f( 1.0f, 1.0f, 1.0f); // Top Right Of The Texture and Quad
            glVertex3f(-1.0f, 1.0f, 1.0f); // Top Left Of The Texture and Quad
    glEnd();

    for (iter = modelObjsList->begin(); iter != modelObjsList->end(); iter++) {
        if ( strcmp( ( (IFObjHeader *)*iter )->getObjType(), "ball") == 0 ) {
            glTranslatef( ( (Ball *)*iter )->getLocation().getX(),
                          ( (Ball *)*iter )->getLocation().getY(),
                          ( (Ball *)*iter )->getLocation().getZ() );
            gluSphere(quadricObj, ( (Ball *)*iter )->getR(), 32, 32);
        }
        if ( strcmp( ( (IFObjHeader *)*iter )->getObjType(), "sheet") == 0 ) {
            qDebug("aa");

        }
    }



//    xrot += xspeed; // X-axis rotation
//    yrot += yspeed; // Y-axis rotation

    // Show message when an enabled OpenGL feature has changed
    glLoadIdentity();

}


// This is called when the OpenGL window is resized
void WidgetSimulationView::resizeGL(int width, int height) {
    // Prevent divide by zero (in the gluPerspective call)
    if (height == 0)
        height = 1;

    glViewport(0, 0, width, height); // Reset current viewport

    glMatrixMode(GL_PROJECTION); // Select projection matrix
    glLoadIdentity(); // Reset projection matrix

    gluPerspective(45.0f, static_cast<GLfloat>(width)/height, 0.1f, 100.0f); // Calculate aspect ratio

    glMatrixMode(GL_MODELVIEW); // Select modelview matrix
    glLoadIdentity(); // Reset modelview matrix
}

// Draw a cube using OpenGL
void WidgetSimulationView::drawCube() {
    glBegin(GL_QUADS);
        // Front Face
        glNormal3f(0.0f, 0.0f, 1.0f);
        glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f, 1.0f); // Bottom Left Of The Texture and Quad
        glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f, 1.0f); // Bottom Right Of The Texture and Quad
        glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f, 1.0f, 1.0f); // Top Right Of The Texture and Quad
        glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f, 1.0f, 1.0f); // Top Left Of The Texture and Quad
        // Back Face
        glNormal3f(0.0f, 0.0f, -1.0f);
        glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f); // Bottom Right Of The Texture and Quad
        glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f, 1.0f, -1.0f); // Top Right Of The Texture and Quad
        glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f, 1.0f, -1.0f); // Top Left Of The Texture and Quad
        glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f); // Bottom Left Of The Texture and Quad
        // Top Face
        glNormal3f(0.0f, 1.0f, 0.0f);
        glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f, 1.0f, -1.0f); // Top Left Of The Texture and Quad
        glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, 1.0f, 1.0f); // Bottom Left Of The Texture and Quad
        glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, 1.0f, 1.0f); // Bottom Right Of The Texture and Quad
        glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f, 1.0f, -1.0f); // Top Right Of The Texture and Quad
        // Bottom Face
        glNormal3f(0.0f, -1.0f, 0.0f);
        glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f, -1.0f, -1.0f); // Top Right Of The Texture and Quad
        glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f, -1.0f, -1.0f); // Top Left Of The Texture and Quad
        glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f, 1.0f); // Bottom Left Of The Texture and Quad
        glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f, 1.0f); // Bottom Right Of The Texture and Quad
        // Right face
        glNormal3f(1.0f, 0.0f, 0.0f);
        glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f); // Bottom Right Of The Texture and Quad
        glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f, 1.0f, -1.0f); // Top Right Of The Texture and Quad
        glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f, 1.0f, 1.0f); // Top Left Of The Texture and Quad
        glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f, 1.0f); // Bottom Left Of The Texture and Quad
        // Left Face
        glNormal3f(-1.0f, 0.0f, 0.0f);
        glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f); // Bottom Left Of The Texture and Quad
        glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f, 1.0f); // Bottom Right Of The Texture and Quad
        glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f, 1.0f, 1.0f); // Top Right Of The Texture and Quad
        glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f, 1.0f, -1.0f); // Top Left Of The Texture and Quad
    glEnd();
}
